/*
ONE BEER license agreement:
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation.
   This program is distributed in the hope that it will be useful,
   but AS IS, WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

   Copyright (C) Jan 2008.
   Author AKA sam_soft

   Revision history
   v1.0.0. Jan 2008.  created
   v1.0.1. June 2008.  forse start added

   Description:
   feed control for MIG MAG
*/ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

// defs and types
#define BYTE unsigned char
#define SETUP 0
#define VALVE_PORT PORTB
#define RELAY_PORT PORTB


#define ENGINE_PORT PORTB
#define ENGINE_BIT PB0
#define VALVE_BIT  PB3
#define RELAY_BIT PB2

#define set_portbit( port, bit ) asm("sbi %0, %1"::"I"(_SFR_IO_ADDR(port)),"I"( bit) )
#define clear_portbit( port, bit ) asm("cbi %0, %1"::"I"(_SFR_IO_ADDR(port)),"I"( bit) )


#define VALVE_ON set_portbit( VALVE_PORT, VALVE_BIT )
#define VALVE_OFF clear_portbit( VALVE_PORT, VALVE_BIT )

#define RELAY_ON set_portbit( RELAY_PORT, RELAY_BIT )
#define RELAY_OFF clear_portbit( RELAY_PORT, RELAY_BIT )
#define ENGINE_SPEED_MAX 245


void Init( void);
void delay_ms( unsigned int ms_delay );
void delay_sec( unsigned char sec_delay );


int adc_setup = 0;

int main (void )
{
	Init();
	

	while(1)
	{
		static unsigned char  cntStart = 0;
		static unsigned char  cntStop = 0;
	

		if( PINB & 0x02 ) // PB1 is button in negative logic
		{
			if( ++cntStop > 250 )// noise canceler
			{
				OCR0A = 0;
				RELAY_OFF;
				delay_ms( 200 );
				delay_ms( 200 );
				VALVE_OFF;									
			}

			cntStart=0;
		}
		else
		{
			static unsigned char engine_speed_nominal = 0;
			static unsigned char engine_speed = 0;
			static unsigned int delay_start = 0;
			if( OCR0A == 0)
			{
				VALVE_ON;
				if( ++cntStart > 250) 
				{
					// force start
					RELAY_ON;
					delay_ms( 200 );
					engine_speed_nominal = ( unsigned char ) (adc_setup >> 2 ) - 10;
					engine_speed = ENGINE_SPEED_MAX;
					// number of ticks is engine speed 
					// start time = delay * ticks
					// so 
					// delay = time to start /  engine_speed
					delay_start = (unsigned char) (500 / ENGINE_SPEED_MAX);
					while( engine_speed > engine_speed_nominal )
					{
						OCR0A = engine_speed;
						delay_ms( delay_start );
						engine_speed--;
					}
					OCR0A = engine_speed_nominal;	
				}
						
			}
	
			cntStop =0;				
		}
	}
	return 0;
}


BYTE start_adc = 1;
BYTE adc_mux =2;


// ISR ADC conversion ready.
ISR( ADC_vect )
{	
	//skip first conversion
	if( start_adc )
	{
		start_adc = 0;
		return;
	}
	adc_setup =  ( ADCW + adc_setup ) >> 1;
}

unsigned int volatile timer_delay = 0;
void delay_ms( unsigned int delay )
{
	timer_delay = 	delay * 38;
	while( timer_delay );
}



ISR( TIM0_OVF_vect )
{
 // 37.5 kHz , so one tick aprox each 26.6 us
 if( timer_delay > 0 )
 {
	timer_delay--;
 }
}


void Init( void )
{
	DDRB |= ( 1<< VALVE_BIT) | (1 <<  ENGINE_BIT) | (1 <<  RELAY_BIT ); // PWM engine, breake relay and  gas valve are output
	// setup ADC 
	// 9.6Mhz / 128 = 75kHz  then / 13 per sample / 
	// 5769 samples 

	ADCSRA |= ( 1<< ADEN ) | ( 1<< ADATE )| ( 1<< ADIE ) | ( 1<< ADPS2)|( 1<< ADPS1 )|( 1<< ADPS0 ) ; 
	ADMUX = adc_mux;

	// PWM timer
	TCCR0A = 0x83;// fast pwm mode with clear on compare
	OCR0A = 0; // initial value is 0
	TIMSK0 |= _BV( TOIE0 );// timer over interrupt
	TCCR0B = 0x01; // start timer with no prescaler - 37.5 khz
	sei();
	// start ADC
	ADCSRA |= ( 1 <<  ADSC );
}
